#!/bin/sh

twoToThePowerOf() {
	count=$1
	value=1
	while [ $count -gt 0 ]
	do
		value=`expr $value '*' 2`
		count=`expr $count - 1`
	done
	echo $value
}

makeImage() {
	namewanted="$1"
	id="$2"
	patientname="$3"
	comments="$4"
	bits="$5"
	min="$6"
	max="$7"
	signed="$8"
	intercept="$9"
	center="${10}"
	width="${11}"
	
	foldername="IMAGES/${namewanted}"
	mkdir -p "${foldername}"
	filename="${foldername}/${namewanted}.dcm"
	usedate=`date '+%Y%m%d'`
	usetime=`date '+%H%M%S'`
	
	echo "$filename"
	if [ -z "$center" -o  -z "$width" ]
	then
		if [ -z "$intercept" ]
		then
			dcsmpte "$filename" -bits $bits -minval " $min" -maxval " $max" $signed \
				-nodisclaimer \
				-r PatientID "$id" -r PatientName "$patientname" \
				-r StudyID "$id" \
				-r StudyDate "${usedate}" -r StudyTime "${usetime}" \
				-r SeriesNumber 1 \
				-r InstanceNumber 1 \
				-r ImageComments "$comments" \
				-r BodyPartExamined CHEST
		else
			dcsmpte "$filename" -bits $bits -minval " $min" -maxval " $max" $signed \
				-nodisclaimer \
				-r RescaleSlope "1" -r RescaleIntercept " $intercept" -r RescaleType US \
				-r PatientID "$id" -r PatientName "$patientname" \
				-r StudyID "$id" \
				-r StudyDate "${usedate}" -r StudyTime "${usetime}" \
				-r SeriesNumber 1 \
				-r InstanceNumber 1 \
				-r ImageComments "$comments" \
				-r BodyPartExamined CHEST
		fi
	else
		if [ -z "$intercept" ]
		then
			dcsmpte "$filename" -bits $bits -minval " $min" -maxval " $max" $signed \
				-nodisclaimer \
				-r WindowCenter " $center" -r WindowWidth " $width" \
				-r PatientID "$id" -r PatientName "$patientname" \
				-r StudyID "$id" \
				-r StudyDate "${usedate}" -r StudyTime "${usetime}" \
				-r SeriesNumber 1 \
				-r InstanceNumber 1 \
				-r ImageComments "$comments" \
				-r BodyPartExamined CHEST
		else
			dcsmpte "$filename" -bits $bits -minval " $min" -maxval " $max" $signed \
				-nodisclaimer \
				-r WindowCenter " $center" -r WindowWidth " $width" \
				-r RescaleSlope "1" -r RescaleIntercept " $intercept" -r RescaleType US \
				-r PatientID "$id" -r PatientName "$patientname" \
				-r StudyID "$id" \
				-r StudyDate "${usedate}" -r StudyTime "${usetime}" \
				-r SeriesNumber 1 \
				-r InstanceNumber 1 \
				-r ImageComments "$comments" \
				-r BodyPartExamined CHEST
		fi
	fi
	dctable -describe -k BitsStored -k BitsAllocated -k HighBit -k WindowCenter -k WindowWidth -k RescaleIntercept "$filename"
	dcdump "$filename" 2>&1 | grep 'Window'
	dcstats "$filename"
	dchist "$filename"
	dciodvfy "$filename"
	echo
}

for bits in 9 10 11 12 13 14 15 16
do
	# Unsigned
	
	min=0
	range=`twoToThePowerOf $bits`
	max=`expr $range - 1`

	id="${bits}unsignedNW"
	patientname="Unsigned^$bits No Window"
	comments="$bits bit unsigned $min to $max, No Window"
	filename="test${bits}unsignedNW"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "" ""
	
	bitsforcenter=`expr $bits - 1`
	center=`twoToThePowerOf $bitsforcenter`
	center=`expr $center - 1`
	width=$range

	id="${bits}unsigned"
	patientname="Unsigned^$bits"
	comments="$bits bit unsigned $min to $max, Window $center $width"
	filename="test${bits}unsigned"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "" "" "$center" "$width"

	# Unsigned, with limited positive values
	
	min=0
	range=`twoToThePowerOf $bits`
	max=`expr $range - 1`
	if [ $max -gt 2047 ]
	then
		max=2047
	fi

	id="${bits}unsignedLMNW"
	patientname="UnsignedLimited^$bits No Window"
	comments="$bits bit unsigned $min to $max, No Window"
	filename="test${bits}unsignedLimitedNW"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "" ""

	center=`expr $min + $max`
	center=`expr $center / 2`	# should really be + 0.5 in most cases
	width=`expr $max - $min`
	width=`expr $width + 1`

	id="${bits}unsignedLM"
	patientname="UnsignedLimited^$bits"
	comments="$bits bit unsigned $min to $max, Window $center $width"
	filename="test${bits}unsignedLimited"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "" "" "$center" "$width"

	# Unsigned, with limited positive values and rescale intercept
	
	min=0
	range=`twoToThePowerOf $bits`
	max=`expr $range - 1`
	if [ $max -gt 2047 ]
	then
		max=2047
	fi

	intercept=-1024

	id="${bits}unsignedLMRINW"
	patientname="UnsignedLimited^$bits No Window"
	comments="$bits bit unsigned $min to $max, No Window, Rescale Intercept $intercept"
	filename="test${bits}unsignedLimitedRescaleNW"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "" "$intercept"

	center=`expr $min + $max`
	center=`expr $center / 2`	# should really be + 0.5 in most cases
	center=`expr $center + $intercept`
	width=`expr $max - $min`
	width=`expr $width + 1`

	id="${bits}unsignedLMRI"
	patientname="UnsignedLimitedRescale^$bits"
	comments="$bits bit unsigned $min to $max, Window $center $width, Rescale Intercept $intercept"
	filename="test${bits}unsignedLimitedRescale"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "" "$intercept" "$center" "$width"

	# Signed, with full range of pixel values used
	
	min=`expr $range / 2`
	min=`expr 0 - $min`
	max=`expr $range / 2`
	max=`expr $max - 1`

	id="${bits}signedNW"
	patientname="Signed^$bits No Window"
	comments="$bits bit signed $min to $max, No Window"
	filename="test${bits}signedNW"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" ""

	center=0
	width=$range

	id="${bits}signed"
	patientname="Signed^$bits"
	comments="$bits bit signed $min to $max, Window $center $width"
	filename="test${bits}signed"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" "" "$center" "$width"

	# Signed, with only positive values

	min=0
	max=`expr $range / 2`
	max=`expr $max - 1`

	id="${bits}signedPNW"
	patientname="SignedPositive^$bits No Window"
	comments="$bits bit signed $min to $max, No Window"
	filename="test${bits}signedPositiveNW"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" ""

	center=`expr $min + $max`
	center=`expr $center / 2`	# should really be + 0.5 in most cases
	width=`expr $max - $min`
	width=`expr $width + 1`

	id="${bits}signedP"
	patientname="SignedPositive^$bits"
	comments="$bits bit signed $min to $max, Window $center $width"
	filename="test${bits}signedPositive"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" "" "$center" "$width"

	# Signed, with only positive values, but rescale intercept

	min=0
	max=`expr $range / 2`
	max=`expr $max - 1`
	
	intercept=-1024

	id="${bits}signedPRINW"
	patientname="SignedPositiveRescale^$bits No Window"
	comments="$bits bit signed $min to $max, No Window, Rescale Intercept $intercept"
	filename="test${bits}signedPositiveRescaleNW"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" "$intercept"

	center=`expr $min + $max`
	center=`expr $center / 2`	# should really be + 0.5 in most cases
	center=`expr $center + $intercept`
	width=`expr $max - $min`
	width=`expr $width + 1`

	id="${bits}signedP"
	patientname="SignedPositiveRescale^$bits"
	comments="$bits bit signed $min to $max, Window $center $width, Rescale Intercept $intercept"
	filename="test${bits}signedPositiveRescale"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" "$intercept" "$center" "$width"

	# Signed, with small negative values

	min=-128
	max=`expr $range / 2`
	max=`expr $max - 1`

	id="${bits}signedSNNW"
	patientname="SignedSmallNegative^$bits No Window"
	comments="$bits bit signed $min to $max, No Window"
	filename="test${bits}signedSmallNegativeNW"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" ""

	center=`expr $min + $max`
	center=`expr $center / 2`	# should really be + 0.5 in most cases
	width=`expr $max - $min`
	width=`expr $width + 1`

	id="${bits}signedSN"
	patientname="SignedSmallNegative^$bits"
	comments="$bits bit signed $min to $max, Window $center $width"
	filename="test${bits}signedSmallNegative"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" "" "$center" "$width"

	# Signed, with small negative values and limited positive values (i.e. typical of CT)

	min=`expr $range / 2`
	min=`expr 0 - $min`
	if [ $min -lt -1024 ]
	then
		min=-1024
	fi
	max=`expr $range / 2`
	max=`expr $max - 1`
	if [ $max -gt 1023 ]
	then
		max=1023
	fi

	id="${bits}signedLMNW"
	patientname="SignedLimited^$bits No Window"
	comments="$bits bit signed $min to $max, No Window"
	filename="test${bits}signedLimitedNW"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" ""

	center=`expr $min + $max`
	center=`expr $center / 2`	# should really be + 0.5 in most cases
	width=`expr $max - $min`
	width=`expr $width + 1`

	id="${bits}signedLM"
	patientname="SignedLimited^$bits"
	comments="$bits bit signed $min to $max, Window $center $width"
	filename="test${bits}signedLimited"
	makeImage "$filename" "$id" "$patientname" "$comments" "$bits" "$min" "$max" "-signed" "" "$center" "$width"

done
	